@@ -10,56 +10,64 @@
  ***************************************************************************/
 	 
 #include "common.h"
-#include "helpers.h"
+#include "helper.h"
 #include "memtypes.h"
+#include "atcommand.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
 
 inline char* get_vendor (void) {
-  return get_value("AT+CGMI");
+  at_command_send(AT_GEN_VENDOR,NULL);
+  return at_get_value(AT_GEN_VENDOR);
 }
 
 inline char* get_model (void) {
-  return get_value("AT+CGMM");
+  at_command_send(AT_GEN_MODEL,NULL);
+  return at_get_value(AT_GEN_MODEL);
 }
 
 inline char* get_revision (void) {
-  return get_value("AT+CGMR");
+  at_command_send(AT_GEN_REVISION,NULL);
+  return at_get_value(AT_GEN_REVISION);
 }
 
 inline char* get_phoneserial (void) {
-  return get_value("AT+CGSN");
+  at_command_send(AT_GEN_IMSI,NULL);
+  return at_get_value(AT_GEN_IMSI);
 }
 
 inline char* get_simserial (void) {
-  return get_value("AT+CIMI");
+  at_command_send(AT_GEN_IMEI,NULL);
+  return at_get_value(AT_GEN_IMEI);
 }
 
 int pbook_select_mem (char* mem) {
   char* command;
   char* temp;
-  int retval;
+  enum return_code status;
 
-  command = mem_alloc(sizeof(AT_PREFIX)-1 + 5
-		      + 2 + str_len(mem) + 2, 1);
-  temp=get_vendor();
-  if (temp!=NULL && !strcmp(temp,"SIEMENS")) {
-    new_at_command(command,"^SPBS=");
+  if (strcasecmp(mem,VCARD_PHONEBOOK_NAME) == 0) return 1;
+  temp = get_vendor();
+  if (temp != NULL && strcmp(temp,"SIEMENS") == 0) {
+    at_sie_phonebook_select(mem);
+    command = AT_SIE_PB_SELECT;
   } else {
-    new_at_command(command,"+CPBS=");
+    at_gen_phonebook_select(mem);
+    command = AT_GEN_PB_SELECT;
   }
-  add_at_command(command,mem);
-  retval = verify_command(command,NULL,0);
-  mem_realloc(command,0);
-  return retval;
+  mem_realloc(temp,0);
+  temp = at_read_line();
+  status = at_line_type(temp,NULL,NULL);
+  mem_realloc(temp,0);
+  if (status == AT_RET_OK) return 1;
+  else return 0;
 }
 
 int sms_select_mem (char* mem, struct slot_range* r,
 		    int* current_fill)
 {
-  char* command;
   char* temp;
   char* ausgabe;
   int retval = 0;
@@ -67,15 +75,11 @@ int sms_select_mem (char* mem, struct slot_range* r,
   if (str_len(mem) == 0) {
     return 0;
   }
-  command = mem_alloc(sizeof(AT_PREFIX)-1 + 5 + 1
-		      + 2*(str_len(mem)+2) + 2, 1);
-  new_at_command(command,"+CPMS");
-  add_at_command(command,"=\"%s\",\"%s\"",mem,mem);
-  ausgabe = get_value(command);
+
+  at_gen_sms_mem(mem,mem,NULL);
+  ausgabe = at_get_value(AT_GEN_SMS_MEM);
   //example: +CPMS: 10,112,10,112,8,12
-  mem_realloc(command,0);
-  if (str_len(ausgabe) != 0 &&
-      strncmp(ausgabe,"+CPMS: ",7) == 0) {
+  if (str_len(ausgabe) != 0) {
     if (r != NULL) {
       strncpy(r->name,mem,sizeof(r->name));
       temp = index(ausgabe,',');
@@ -84,8 +88,8 @@ int sms_select_mem (char* mem, struct slot_range* r,
 	r->max = atoi(temp+1);
       }
     }
-    if (current_fill != NULL && str_len(ausgabe+7)) {
-      *current_fill = atoi(ausgabe+7);
+    if (current_fill != NULL && str_len(ausgabe)) {
+      *current_fill = atoi(ausgabe);
     }
     retval = 1;
   }
@@ -93,52 +97,67 @@ int sms_select_mem (char* mem, struct slot_range* r,
   return retval;
 }
 
+#include "ttyaccess.h"
 int phone_init (void) {
-  int retval;
-  char* command = "ATZ";
+  int retval = 1;
   char* ack;
 
-  tty_write_command("");
-  retval = verify_command(command,NULL,0);
-  if (retval < 0) { //timeout
-    ack = tty_read(command);
-    retval = check_return_code(ack,NULL,0);
+  tty_write("\r",1);
+  at_command_send(AT_GEN_INIT,NULL);
+  ack = at_read_line();
+  if (at_line_type(ack,NULL,NULL) != AT_RET_OK) {
     mem_realloc(ack,0);
-    if (retval < 0) {
+    ack = at_read_line();
+    if (at_line_type(ack,NULL,NULL) != AT_RET_OK) {
       retval = 0;
     }
+    mem_realloc(ack,0);
   }
   return retval;;
 }
 
 int command_echo (int enable) {
   char* command;
+  char* ack;
+  enum return_code status;
   if (enable) {
-    command = "ATE1";
+    command = AT_GEN_ECHO_ENABLE;
   } else {
-    command = "ATE0";
+    command = AT_GEN_ECHO_DISABLE;
   }
-  return verify_command(command,NULL,0);
+  at_command_send(command,NULL);
+  ack = at_read_line();
+  status = at_line_type(ack,NULL,NULL);
+  mem_realloc(ack,0);
+  return (status == AT_RET_OK);
 }
 
 int verbose_errors (int enable) {
-  char* command;
+  char* parmlist;
+  char* ack;
+  enum return_code status;
   if (enable) {
-    command = "AT+CMEE=2";
+    parmlist = "2";
   } else {
-    command = "AT+CMEE=0";
+    parmlist = "0";
   }
-  return verify_command(command,NULL,0);
+  at_command_send(AT_GEN_ERROR,parmlist);
+  ack = at_read_line();
+  status = at_line_type(ack,NULL,NULL);
+  mem_realloc(ack,0);
+  return (status == AT_RET_OK);
 }
 
 char* get_charset (void) {
   char* ausgabe;
   char* temp;
     
-  ausgabe=get_value("AT+CSCS?");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CSCS: \"",8)) {
-    if ((temp=index(ausgabe+8,'"'))!=NULL) {
-      temp=strn_dup(ausgabe+8,temp-ausgabe-8);
+  at_syntax_request(AT_GEN_CHARSET);
+  ausgabe = at_get_value(AT_GEN_CHARSET);
+  if (ausgabe != NULL) {
+    if (*ausgabe == '"' && 
+	(temp = index(ausgabe,'"')) != NULL) {
+      temp = strn_dup(ausgabe+1,temp-ausgabe-1);
       mem_realloc(ausgabe,0);
       return temp;
     }
@@ -149,68 +168,67 @@ char* get_charset (void) {
 char** get_charsets (void) {
   char* ausgabe;
   char** retval;
-  int p;
 
-  ausgabe=get_value("AT+CSCS=?");
-  p = strlen(AT_REPLY_PREFIX(AT_CHARSET));
+  at_command_send(AT_GEN_CHARSET,"?");
+  ausgabe = at_get_value(AT_GEN_CHARSET);
   //example: +CSCS: ("GSM","UCS2")
-  if (str_len(ausgabe) &&
-      !strncmp(ausgabe,AT_REPLY_PREFIX(AT_CHARSET),p)) {
-    retval = parse_at_stringlist(ausgabe+p);
+  if (str_len(ausgabe) != 0) {
+    retval = at_parse_stringlist(ausgabe);
   } else {
     retval = NULL;
   }
+  mem_realloc(ausgabe,0);
   return retval;
 }
 
 int set_charset(char *charset) {
-  char *command;
-  int  retval;
+  char *temp;
+  int  retval = 0;
 
-  if (!str_len(charset)) {
-    errexit ("No charset specified.\n");
+  if (str_len(charset) == 0) {
+    errexit ("%s","Error: no charset specified.\n");
   }
-  command = mem_alloc(sizeof(AT_PREFIX)-1
-		      + sizeof(AT_CHARSET)-1
-		      + 2 + str_len(charset) + 2, 1);
-  new_at_command(command, AT_CHARSET);
-  add_at_command(command, "=\"%s\"",charset);
-  retval = verify_command(command,NULL,0);
-  mem_realloc(command,0);
+  at_gen_charset(charset);
+  temp = at_read_line();
+  if (at_line_type(temp,NULL,NULL) == AT_RET_OK) {
+    retval = 1;
+  }
+  mem_realloc(temp,0);
   return retval;
 }
 
 void set_smsc (char* number) {
-  char* command;
-  char error[128];
+  char* temp;
+  char* error;
+  enum return_code status;
 
-  if (!str_len(number)) {
-    errexit("No number specified.\n");
-  } else if (!is_pnumber(number,1)) {
-    errexit("No valid number specified.\n");
+  if (str_len(number) == 0) {
+    errexit("%s","Error: no number specified.\n");
+  } else if (is_telephone_number(number) == 0) {
+    errexit("%s","Error: no valid number specified.\n");
   }
-  command = mem_alloc(sizeof(AT_PREFIX)-1
-		      + sizeof(AT_SMSC)-1
-		      + 2 + str_len(number) + 2, 1);
-  new_at_command(command,AT_SMSC);
-  add_at_command(command,"=\"%s\",%d",number,numtype(number));
-
-  if (verify_command(command,error,sizeof(error))) {
+  at_gen_smsc(number);
+  temp = at_read_line();
+  status = at_line_type(temp,AT_GEN_SMSC,&error);
+  if (status == AT_RET_OK) {
     myprintf(0,"SMS server number was set to %s\n",number);
+  } else if (status == AT_RET_ERROR) {
+    errexit("%s","Error: unknown cause\n");
   } else {
     errexit("Error: %s\n",error);
   }
-  mem_realloc(command,0);
+  mem_realloc(temp,0);
 }
 
 char* get_smsc (void) {
   char* ausgabe;
   char* temp;
     
-  ausgabe=get_value("AT+CSCA?");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CSCA: \"",8)) {
-    if ((temp=strstr(ausgabe+8,"\","))!=NULL) {
-      temp=strn_dup(ausgabe+8,temp-ausgabe-8);
+  at_syntax_request(AT_GEN_SMSC);
+  ausgabe = at_get_value(AT_GEN_SMSC);
+  if (ausgabe != NULL && *ausgabe == '"') {
+    if ((temp=strstr(ausgabe,"\",")) != NULL) {
+      temp = strn_dup(ausgabe+1,temp-ausgabe-1);
       mem_realloc(ausgabe,0);
       return temp;
     }
@@ -221,24 +239,21 @@ char* get_smsc (void) {
 #include "timeincl.h"
 
 void set_time (void) {
-  char at_command[128];
   char* ack;
   time_t seconds;
-  char timestr[128];
+  char timestr[20]; //6*2 + 7 + 1
 	
-  memset(at_command,0,sizeof(at_command));
   memset(timestr,0,sizeof(timestr));
 	
-  new_at_command(at_command,"+CCLK");
   time(&seconds); //seconds since Unix
   //string formatting with conversion to local time
-  strftime(timestr,sizeof(timestr),"%y/%m/%d,%H:%M:%S",localtime(&seconds));
-  add_at_command(at_command,"=\"%s\"",timestr);
-  ack=tty_write_read(at_command);
-  if (!strcmp(ack,"OK")) {
-    myprintf(0,"Time was synchronized\n");
+  strftime(timestr,sizeof(timestr),"\"%y/%m/%d,%H:%M:%S\"",localtime(&seconds));
+  at_command_send(AT_GEN_TIME,timestr);
+  ack = at_read_line();
+  if (at_line_type(ack,NULL,NULL) == AT_RET_OK) {
+    myprintf(0,"%s","Time was synchronized\n");
   } else {
-    errexit("Time could not be synchronized\n");
+    errexit("%s","Time could not be synchronized\n");
   }
   mem_realloc(ack,0);
 }
@@ -246,15 +261,18 @@ void set_time (void) {
 struct tm* get_time (void) {
   char* ausgabe;
   struct tm* phonetime;
+  char* status;
   
-  ausgabe=get_value("AT+CCLK?");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CCLK: \"",8)) {
-    phonetime=mem_alloc(sizeof(struct tm),1);
-    if (strptime(ausgabe+8,"%y/%m/%d,%H:%M:%S",phonetime)!=NULL) {
-      mem_realloc(ausgabe,0);
+  at_syntax_request(AT_GEN_TIME);
+  ausgabe = at_get_value(AT_GEN_TIME);
+  if (ausgabe != NULL) {
+    //example: +CCLK: "04/06/10,02:30:37"
+    phonetime = mem_alloc(sizeof(*phonetime),1);
+    status = strptime(ausgabe,"\"%y/%m/%d,%H:%M:%S\"",phonetime);
+    mem_realloc(ausgabe,0);
+    if (status != NULL) {
       return phonetime;
     }
-    mem_realloc(ausgabe,0);
     mem_realloc(phonetime,0);
   }
   return NULL;
@@ -263,13 +281,11 @@ struct tm* get_time (void) {
 char* get_simid (void) {
   char* ausgabe;
 
-  ausgabe=get_vendor();
-  if(ausgabe!=NULL && !strcasecmp(ausgabe,"SIEMENS")) {
-    ausgabe=get_value("AT^SCID");
-    if (ausgabe!=NULL && !strncmp(ausgabe,"^SCID: ",7)) {
-      memmove(ausgabe,ausgabe+7,strlen(ausgabe+7)+1);
-      return mem_realloc(ausgabe,strlen(ausgabe)+1);
-    }
+  ausgabe = get_vendor();
+  if(ausgabe != NULL &&
+     strcasecmp(ausgabe,"SIEMENS") == 0) {
+    at_command_send(AT_SIE_CARD_ID,NULL);
+    return at_get_value(AT_SIE_CARD_ID);
   }
   return NULL;
 }
@@ -278,17 +294,16 @@ char* get_operator (void) {
   char* ausgabe;
   char* temp;
 
-  ausgabe=get_value("AT+COPS?");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+COPS: ",7)) {
-    if ((temp=index(ausgabe+7,'"'))!=NULL) {
-      ++temp;
-      if (temp[strlen(temp)-1]=='"') {
-	temp[strlen(temp)-1]=0;
-      }
-      temp=strn_dup(temp,strlen(temp));
+  at_syntax_request(AT_GEN_OPERATOR);
+  ausgabe = at_get_value(AT_GEN_OPERATOR);
+  if (ausgabe != NULL) {
+    //example: +COPS: 0,0,"E-Plus"
+    if ((temp=index(ausgabe,'"')) != NULL) {
+      temp = strn_dup(temp+1,strlen(temp+1)-1);
       mem_realloc(ausgabe,0);
       return temp;
     }
+    mem_realloc(ausgabe,0);
   }
   return NULL;
 }
@@ -297,13 +312,16 @@ char* get_battery (void) {
   char* ausgabe;
   char* temp;
     
-  ausgabe=get_value("AT+CBC");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CBC: ",6)) {
-    if ((temp=index(ausgabe+6,','))!=NULL) {
+  at_command_send(AT_GEN_BATTERY,NULL);
+  ausgabe = at_get_value(AT_GEN_BATTERY);
+  if (ausgabe != NULL) {
+    //example: +CBC: 0,80
+    if ((temp=index(ausgabe+6,',')) != NULL) {
       temp=strn_dup(temp+1,strlen(temp+1));
       mem_realloc(ausgabe,0);
       return temp;
     }
+    mem_realloc(ausgabe,0);
   }
   return NULL;
 }
@@ -312,31 +330,31 @@ int get_signal (int* ber_p) {
   char* ausgabe;
   char* temp;
   int signal = -1;
+  int b;
+  
 
   if (ber_p != NULL) {
     *ber_p = -1;
   }    
-  ausgabe=get_value("AT+CSQ");
-  //example: +CSQ: 28,99
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CSQ: ",6)) {
-    signal=atoi(ausgabe+6);
-    if (signal==99) {
-      signal=-1;
+  at_command_send(AT_GEN_SIGNAL,NULL);
+  ausgabe = at_get_value(AT_GEN_SIGNAL);
+  if (ausgabe != NULL) {
+    //example: +CSQ: 28,99
+    signal = atoi(ausgabe+6);
+    if (signal == 99) {
+      signal = -1;
     } else {
       if ((signal=111-((signal-1)*2)) < 0) {
 	signal = -1;
       }
     }
     if (ber_p != NULL) {
-      if ((temp=index(ausgabe,','))!=NULL) {
-	*ber_p=atoi(temp+1);
-	if (*ber_p==7) {
-	  *ber_p=255;
-	} else if (*ber_p>=0 && *ber_p <7) {
-	  *ber_p=2<<(*ber_p);
-	} else {
-	  // unknown value (the standard 99 means the same)
-	  *ber_p=-1;
+      if ((temp=index(ausgabe,',')) != NULL) {
+	b = atoi(temp+1);
+	if (b == 7) {
+	  *ber_p = 255;
+	} else if (b >= 0 && b < 7) {
+	  *ber_p = 2<<b;
 	}
       }
     }
@@ -349,59 +367,65 @@ char* get_netstatus (unsigned int* areacode_p, unsigned int* cellid_p) {
   char* ausgabe;
   char* index1;
   char* index2;
+  char buffer;
 
-  ausgabe=get_value("AT+CREG=?");
-  if (index(ausgabe,'2')==NULL) {
+  if (areacode_p != NULL) *areacode_p=0;
+  if (cellid_p != NULL) *cellid_p=0;
+
+  at_command_send(AT_GEN_CSD_STATUS,"?");
+  ausgabe = at_get_value(AT_GEN_CSD_STATUS);
+  //example: +CREG: (0-2)
+  if (ausgabe != NULL && index(ausgabe,'2') == NULL) {
     mem_realloc(ausgabe,0);
-    ausgabe=get_value("AT+CREG?");
+    at_syntax_request(AT_GEN_CSD_STATUS);
+    ausgabe = at_get_value(AT_GEN_CSD_STATUS);
   } else {
     mem_realloc(ausgabe,0);
-    /* Fix: firmware may forget to send LAC/IC (ME45 v23)
-     * Provided by Dominik Neubauer
+    /* Fix: firmware may forget to send LAC/IC (e.g. ME45 v23,S55)
+     * Original was provided by Dominik Neubauer
      */
-    ausgabe=get_value("AT+CREG=2;+CREG?;+CREG=0");
+    at_command_send(AT_GEN_CSD_STATUS,"2");
+    ausgabe = at_read_line();
+    if (at_line_type(ausgabe,NULL,NULL) == AT_RET_OK) {
+      mem_realloc(ausgabe,0);
+      at_syntax_request(AT_GEN_CSD_STATUS);
+      ausgabe = at_get_value(AT_GEN_CSD_STATUS);
+      if (ausgabe != NULL) {
+	at_command_send(AT_GEN_CSD_STATUS,"0");
+	mem_realloc(at_read_line(),0);
+      }
+    } else {
+      ausgabe = mem_realloc(ausgabe,0);
+    }
   }
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CREG: ",7)) {
-    if (areacode_p!=NULL) {
-      *areacode_p=0;
-    }
-    if (areacode_p!=NULL) {
-      *cellid_p=0;
-    }
-    index1=index(ausgabe+7,'"');
-    if (index1!=NULL) {
-      index2=index(++index1,'"');
-      if (index2!=NULL && areacode_p!=NULL) {
-	*areacode_p=hexstr2int(index1,index2-index1);
-	index1=index(index2+1,'"');
-	if (index1!=NULL) {
-	  index2=index(++index1,'"');
-	  if (index2!=NULL && cellid_p!=NULL) {
+  //example: +CREG: 2,1,"0044","6E30"
+  if (ausgabe != NULL) {
+    index1 = index(ausgabe,'"');
+    if (index1 != NULL) {
+      index2 = index(++index1,'"');
+      if (index2 != NULL) {
+	if (areacode_p != NULL) {
+	  *areacode_p = hexstr2int(index1,index2-index1);
+	}
+	index1 = index(index2+1,'"');
+	if (index1 != NULL) {
+	  index2 = index(++index1,'"');
+	  if (index2 != NULL && cellid_p != NULL) {
 	    *cellid_p=hexstr2int(index1,index2-index1);
 	  }
 	}
       }
     }
-    if ((index1=index(ausgabe+7,','))!=NULL) {
-      switch (*(index1+1)) {
-      case '0':
-	mem_realloc(ausgabe,0);
-	return "not checked in, not seeking";
-      case '1':
-	mem_realloc(ausgabe,0);
-	return "checked in";
-      case '2':
-	mem_realloc(ausgabe,0);
-	return "not checked in, but seeking a network";
-      case '3':
-	mem_realloc(ausgabe,0);
-	return "check-in denied by network";
-      case '5':
-	mem_realloc(ausgabe,0);
-	return "registered, roaming";
-      default:
-	mem_realloc(ausgabe,0);
-	return "unknown status";
+    if ((index1=index(ausgabe,',')) != NULL) {
+      buffer = index1[1];
+      mem_realloc(ausgabe,0);
+      switch (buffer) {
+      case '0':	return "not checked in, not seeking";
+      case '1':	return "checked in";
+      case '2':	return "not checked in, but seeking a network";
+      case '3':	return "check-in denied by network";
+      case '5':	return "registered, roaming";
+      default:	return "unknown status";
       }
     }
   }
@@ -412,49 +436,40 @@ char* get_gprs_status (unsigned int* areacode_p, unsigned int* cellid_p) {
   char* ausgabe;
   char* index1;
   char* index2;
+  char buffer;
 
-  ausgabe=get_value("AT+CGREG?");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CGREG: ",8)) {
-    if (areacode_p!=NULL) {
-      *areacode_p=0;
-    }
-    if (areacode_p!=NULL) {
-      *cellid_p=0;
-    }
-    index1=index(ausgabe+8,'"');
-    if (index1!=NULL) {
-      index2=index(++index1,'"');
-      if (index2!=NULL && areacode_p!=NULL) {
-	*areacode_p=hexstr2int(index1,index2-index1);
-	index1=index(index2+1,'"');
-	if (index1!=NULL) {
-	  index2=index(++index1,'"');
-	  if (index2!=NULL && cellid_p!=NULL) {
-	    *cellid_p=hexstr2int(index1,index2-index1);
+  if (areacode_p != NULL) *areacode_p=0;
+  if (cellid_p != NULL) *cellid_p=0;
+
+  at_syntax_request(AT_GEN_GPRS_STATUS);
+  ausgabe = at_get_value(AT_GEN_GPRS_STATUS);
+  if (ausgabe != NULL) {
+    index1 = index(ausgabe,'"');
+    if (index1 != NULL) {
+      index2 = index(++index1,'"');
+      if (index2!=NULL) {
+	if (areacode_p!=NULL) {
+	  *areacode_p = hexstr2int(index1,index2-index1);
+	}
+	index1 = index(index2+1,'"');
+	if (index1 != NULL) {
+	  index2 = index(++index1,'"');
+	  if (index2 != NULL && cellid_p != NULL) {
+	    *cellid_p = hexstr2int(index1,index2-index1);
 	  }
 	}
       }
     }
-    if ((index1=index(ausgabe+8,','))!=NULL) {
-      switch (*(index1+1)) {
-      case '0':
-	mem_realloc(ausgabe,0);
-	return "not registered, not searching";
-      case '1':
-	mem_realloc(ausgabe,0);
-	return "registered, home network";
-      case '2':
-	mem_realloc(ausgabe,0);
-	return "not registered, searching";
-      case '3':
-	mem_realloc(ausgabe,0);
-	return "registration denied by network";
-      case '5':
-	mem_realloc(ausgabe,0);
-	return "registered, roaming";
-      default:
-	mem_realloc(ausgabe,0);
-	return "unknown status";
+    if ((index1=index(ausgabe,',')) != NULL) {
+      buffer = index1[1];
+      mem_realloc(ausgabe,0);
+      switch (buffer) {
+      case '0':	return "not registered, not searching";
+      case '1':	return "registered, home network";
+      case '2':	return "not registered, searching";
+      case '3':	return "registration denied by network";
+      case '5':	return "registered, roaming";
+      default:	return "unknown status";
       }
     }
   }
@@ -463,16 +478,16 @@ char* get_gprs_status (unsigned int* areacode_p, unsigned int* cellid_p) {
 
 int get_gprs_attach (void) {
   char* ausgabe;
+  char buffer;
 
-  ausgabe=get_value("AT+CGATT?");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CGATT: ",8)) {
-    switch (ausgabe[8]) {
-    case '0':
-      mem_realloc(ausgabe,0);
-      return 0;
-    case '1':
-      mem_realloc(ausgabe,0);
-      return 1;
+  at_syntax_request(AT_GEN_GPRS_ATTACH);
+  ausgabe = at_get_value(AT_GEN_GPRS_ATTACH);
+  if (ausgabe != NULL) {
+    buffer = ausgabe[0];
+    mem_realloc(ausgabe,0);
+    switch (buffer) {
+    case '0': return 0;
+    case '1': return 1;
     }
   }
   return -1;
@@ -480,11 +495,19 @@ int get_gprs_attach (void) {
 
 char* get_gprs_class (void) {
   char* ausgabe;
+  char* temp;
 
-  ausgabe=get_value("AT+CGCLASS?");
-  if (ausgabe!=NULL && !strncmp(ausgabe,"+CGCLASS: ",10)) {
-    memmove(ausgabe,ausgabe+10,strlen(ausgabe+10)+1);
-    return mem_realloc(ausgabe,strlen(ausgabe)+1);
+  at_syntax_request(AT_GEN_GPRS_CLASS);
+  ausgabe = at_get_value(AT_GEN_GPRS_CLASS);
+  if (ausgabe != NULL) {
+    //example: +CGCLASS: "B"
+    if (ausgabe[0] == '"') {
+      memmove(ausgabe,ausgabe+1,strlen(ausgabe)+1);
+    }
+    if ((temp=index(ausgabe,'"')) != NULL) {
+      *temp = 0;
+    }
+    return ausgabe;
   }
   return NULL;
 }
@@ -532,7 +555,9 @@ void info_misc (FILE* fd) {
   fprintf(fd,"\nSIM card related information:\n");
   fprintf(fd,"IMSI:\t\t%s\n",temp=get_simserial());
   mem_realloc(temp,0);
-  fprintf(fd,"card ID:\t%s\n",temp=get_simid());
+  if ((temp=get_simid()) != NULL) {
+    fprintf(fd,"card ID:\t%s\n",temp);
+  }
   mem_realloc(temp,0);
 
   fprintf(fd,"\nNetwork related information:\n");
