@@ -10,13 +10,22 @@
  ***************************************************************************/
 
 #include "common.h"
-#include "helpers.h"
+#include "helper.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <string.h>
+#include <unistd.h>
+
+int file_accessible_ro (char* file) {
+  int myfd;
+  myfd = open(file,O_RDONLY);
+  close(myfd);
+  if (myfd == -1) return 0;
+  else return 1;
+}
 
 int open_myFile_ro (char* file) {
   int myfd;
@@ -29,10 +38,10 @@ int open_myFile_ro (char* file) {
     myprintf(0,"Accessing %s\n",file);
   } else {
     myfd=STDIN_FILENO;
-    myprintf(0,"Accessing stdin\n");
+    myprintf(0,"%s","Accessing stdin\n");
   }
   if (isatty(myfd)) {
-    myprintf(0,"Please input the data and finish with Ctrl-D on an empty line:\n");
+    myprintf(0,"%s","Please input the data and finish with Ctrl-D on an empty line:\n");
   }
   return(myfd);
 }
