@@ -1,3 +1,4 @@
+
 /***************************************************************************
  *   copyright           : (C) 2002 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -10,7 +11,7 @@
  ***************************************************************************/
 
 #include "common.h"
-#include "../config.h"
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
@@ -38,6 +39,8 @@ void help(char *me){
   printf(" --pin <pin>             use this pin if one is needed\n");
   printf(" --quiet             -q  decreases verbosity by 1\n");
   printf(" --verbose           -v  increases verbosity by 1\n");
+  printf(" --number <number>   -n  specify number on send\n");
+  printf(" --text <text>       -t  specify content on send\n");
 
   printf("\nOperations:\n");
   printf(" --remove            -r  removes an entry from the phone\n");
@@ -61,10 +64,9 @@ void help(char *me){
 
   printf("\nSMS options:\n");
   printf(" --direct                send/receive without storing in the phone\n");
+  printf(" --sort                  sort short message on getting(type or slot)\n");
 
   printf("\nSMS-PDU creation options:\n");
-  printf(" --number <number>   -n  specify number to send SMS to\n");
-  printf(" --text <text>       -t  specify content of SMS\n");
   printf(" --unicode               use UCS-2 (unicode) as charset\n");
   printf(" --srr                   request a status report from the SMSC\n");
   printf(" --flash                 make the sms appear directly (if supported by\n");
