@@ -9,15 +9,15 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "helpers.h"
+#include "helper.h"
 #include "charsets.h"
 #include <string.h>
 
 char* pb_mask_chars (gsmchar_t* input) {
   char* retval;
-  int i,off;
+  unsigned int i;
+  unsigned int off = 0;
 
-  off=0;
   retval=mem_alloc(strlen(input)*3,1);
   for (i=0; i<strlen(input); i++) {
     switch (*(input+i)&127) {
