@@ -10,11 +10,15 @@
  ***************************************************************************/
 
 #include "common.h"
-#include "helpers.h"
+#include "helper.h"
 #include "options.h"
+#include "atcommand.h"
+#include "ttyaccess.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <unistd.h>
 
 void file_transfer_manage (int   action, char** files,
 			   char* outfile, char* pipe,
@@ -31,21 +35,21 @@ void file_transfer_manage (int   action, char** files,
   int current;
 
   if (mem == NULL) {
-    errexit ("You must define a memory to use.\n");
+    errexit ("%s","You must define a memory to use.\n");
   }
   if (strcmp(mem,"?") == 0) {
     file_print_memlist(NULL,1);
-    exit(0);
+    return;
   }
 
   if (action == 0) {
-    errexit ("You must specify a valid action.\n");
+    errexit ("%s","You must specify a valid action.\n");
   }
 
   if (slot < SCMXX_SLOT_BINARY_MIN &&
       slot != SCMXX_SLOT_UNDEFINED &&
       slot != SCMXX_SLOT_ALL) {
-    errexit ("You must define either a valid slot, nothing or \"all\".\n");
+    errexit ("%s","You must define either a valid slot, nothing or \"all\".\n");
   }
 
   range = NULL;
@@ -55,7 +59,7 @@ void file_transfer_manage (int   action, char** files,
     range = file_get_types(1);
   }
   if (range[i] == NULL) {
-    errexit("ERROR: this phone does not support file exchange.\n");
+    errexit("%s","ERROR: this phone does not support file exchange.\n");
   }
   while (range[i] != NULL &&
 	 strcmp(range[i]->name,mem)) {
@@ -75,7 +79,7 @@ void file_transfer_manage (int   action, char** files,
     
   switch(action){
   default:
-    errexit("You must specify exactly one operation.\n");
+    errexit("%s","You must specify exactly one operation.\n");
     break;
   case SCMXX_ACTION_REMOVE: //deleting
     if (slot == SCMXX_SLOT_ALL) {
@@ -83,7 +87,7 @@ void file_transfer_manage (int   action, char** files,
 	file_delete(mem, current);
       }
     } else if (slot < SCMXX_SLOT_BINARY_MIN) {
-      errexit ("You must define a valid slot or \"all\".\n");
+      errexit ("%s","You must define a valid slot or \"all\".\n");
     } else {
       file_delete(mem, slot);
     }
@@ -145,7 +149,7 @@ void file_transfer_manage (int   action, char** files,
 	  // only empty slots shall be used
 	  current = file_detect_free_slot(mem,current,drange[j]->max);
 	  if (current < 0) {
-	    errexit("No free slot found.\n");
+	    errexit("%s","No free slot found.\n");
 	  } else {
 	    myprintf(0,"Detected empty slot %d\n",current);
 	  }
@@ -171,7 +175,7 @@ void file_transfer_manage (int   action, char** files,
 	    free(filename);
 	  }
 	} else {
-	  errexit("You must specify a file name prefix (can be \"\").\n");
+	  errexit("%s","You must specify a file name prefix (can be \"\").\n");
 	}
       }
     } else {
@@ -182,14 +186,10 @@ void file_transfer_manage (int   action, char** files,
 }
 
 void file_delete (char* ftype, int slot) {
-  char at_command[128];
   char* ausgabe;
 
-  memset(at_command,0,sizeof(at_command));
-
-  new_at_command(at_command,"^SBNW");
-  add_at_command(at_command,"=\"%s\",%d,0",ftype,slot);
-  ausgabe=tty_write_read(at_command);
+  at_sie_binary_write(ftype,slot,0,0);
+  ausgabe = at_read_line();
   if (!strcmp(ausgabe,"OK")) {
     myprintf(0,"%s %d deleted.\n",ftype,slot);
   } else {
@@ -199,47 +199,43 @@ void file_delete (char* ftype, int slot) {
 }
 
 int file_detect_free_slot (char* ftype, int minimum, int maximum) {
-  char at_command[128];
   char* ausgabe;
-  char* ecode;
+  char* content;
   int i;
-  int status;
+  enum return_code status;
     
-  memset(at_command,0,sizeof(at_command));
-     
-  myprintf(0,"Trying to find an empty slot...\n");
+  myprintf(0,"%s","Trying to find an empty slot...\n");
   i = minimum;
   while (i <= maximum) {
-    new_at_command(at_command,"^SBNR");
-    add_at_command(at_command,"=\"%s\",%d",ftype,i);
-    ausgabe = tty_write_read(at_command);
-    ecode = mem_alloc(str_len(ausgabe)+1,1);
-    status = check_return_code(ausgabe,ecode,str_len(ausgabe)+1);
+    at_sie_binary_read(ftype,i);
+    ausgabe = at_read_line();
+    status = at_line_type(ausgabe,AT_SIE_BIN_READ,&content);
     switch (status) {
-    case  -1: 
-      mem_realloc(ecode,0);
+    case AT_RET_OK:
+      mem_realloc(ausgabe,0);
+      return i;
+    case AT_RET_ERROR:
+      errexit("%s","\nError: unknown cause\n");
+    case AT_RET_ERROR_CME:
+    case AT_RET_ERROR_CMS:
+      errexit("\nError: %s\n",content);
+    default: 
       do {
 	mem_realloc(ausgabe,0);
-	ausgabe = tty_read(at_command);
-	status = check_return_code(ausgabe,NULL,0);
-      } while (status < 0);
-      mem_realloc(ausgabe,0);
-      if (status > 0) {
+	ausgabe = at_read_line();
+	status = at_line_type(ausgabe,AT_SIE_BIN_READ,&content);
+      } while (status == AT_RET_ANSWER ||
+	       status == AT_RET_OTHER);
+      if (status == AT_RET_OK) {
 	++i;
+      } else if (status == AT_RET_ERROR) {
+	errexit("%s","\nError: unknown cause\n");
       } else {
-	errexit("\nError: %s\n",ecode);
+	errexit("\nError: %s\n",content);
       }
-      break;
-    case 0:
-      if (strcasecmp(ecode,"unknown") != 0) {
-	errexit("\nError: %s\n",ecode);
-      }
-      //no break
-    case 1:
       mem_realloc(ausgabe,0);
-      mem_realloc(ecode,0);
-      return i;
       break;
+      //no break
     }
   }
   return -1;
@@ -254,7 +250,6 @@ int file_detect_free_slot (char* ftype, int minimum, int maximum) {
 
 #define FILE_PDUSIZE 175
 void file_send (char* file, char* ftype, int slot) {
-  char at_command[128];
   char* ack;
 
   char transfer[(2*FILE_PDUSIZE)+1];
@@ -267,7 +262,6 @@ void file_send (char* file, char* ftype, int slot) {
   unsigned char data;
   char datastr[3];
 
-  memset(at_command,0,sizeof(at_command));
   memset(transfer,0,sizeof(transfer));
   memset(buffer,0,sizeof(buffer));
   memset(datastr,0,sizeof(datastr));
@@ -292,18 +286,16 @@ void file_send (char* file, char* ftype, int slot) {
     }
   }
 
-  myprintf(0,"File transfer...\n");
+  myprintf(0,"%s","File transfer...\n");
   while (strlen(transfer)!=0) {
     //starting packet transfer
-    new_at_command(at_command,"^SBNW");
-    if (strlen(buffer)>0) {
-      add_at_command(at_command,"=\"%s\",%d,%d,%d",ftype,slot,packetnum,packetnum+1);
+    if (strlen(buffer) > 0) {
+      at_sie_binary_write(ftype,slot,packetnum,packetnum+1);
     } else {
-      add_at_command(at_command,"=\"%s\",%d,%d,%d",ftype,slot,packetnum,packetnum);
+      at_sie_binary_write(ftype,slot,packetnum,packetnum);
     }
-    tty_write_command(at_command);
-    myprintf(0,"Waiting for data request...");
-    ack=tty_read(at_command);
+    myprintf(0,"%s","Waiting for data request...");
+    ack = at_read_line();
 
     if (!strncmp(ack,"> ",2)) {
       mem_realloc(ack,0);
@@ -325,20 +317,20 @@ void file_send (char* file, char* ftype, int slot) {
       if (VERBOSE_LEVEL) {
 	myprintf(1,"\nSending %d Bytes in Paket %d\n",lastsent,packetnum);
       } else {
-	myprintf(0,"Sending data...");
+	myprintf(0,"%s","Sending data...");
       }
       //sending hex data
       if (tty_write_data(transfer,2*lastsent) == -1) {
 	close(myfd);
 	errexit("Error on sending data: %s\n", strerror(errno));
       }
-      ack=tty_read(at_command);
+      ack = at_read_line();
       if (!strcmp(ack,"OK")) {
 	myprintf(0,"Packet %d sent\n",packetnum);
       } else {
 	close(myfd);
 	if (!strcmp(ack,"+CME ERROR: INV CHAR IN TEXT")) {
-	  errexit("ERROR: Wrong data format\n");
+	  errexit("%s","ERROR: Wrong data format\n");
 	} else {
 	  errexit("\n\"%s\"\n",ack);
 	}
@@ -368,10 +360,9 @@ void file_send (char* file, char* ftype, int slot) {
     } else {
       if (!strcmp(ack,"+CME ERROR: INVALID INDEX")) {
 	mem_realloc(ack,0);
-	myprintf(0,"ERROR\nPacket buffer seems to be filled with something else, clearing...\n");
-	new_at_command(at_command,"^SBNW");
-	add_at_command(at_command,"=\"%s\",%d",ftype,slot);
-	ack=tty_write_read(at_command);
+	myprintf(0,"%s","ERROR\nPacket buffer seems to be filled with something else, clearing...\n");
+	at_sie_binary_write(ftype,slot,-1,-1);
+	ack = at_read_line();
 	if (strcmp(ack,"OK")) {
 	  errexit("%s, aborting\n",ack);
 	}
@@ -383,43 +374,36 @@ void file_send (char* file, char* ftype, int slot) {
     }
   }
   if (packetnum) {
-    myprintf(0,"File transfer complete.\n");
+    myprintf(0,"%s","File transfer complete.\n");
   } else {
-    myprintf(0,"Nothing to transfer.\n");
+    myprintf(0,"%s","Nothing to transfer.\n");
   }
   close(myfd);
 }
 
 void file_get (char* file, char* ftype, int slot, char* pipe) {
-  char at_command[128];
   char* ausgabe;
   int packetnum=0;
   int maxpackets=0;
   int myfd=-1;
-  int i;
+  unsigned int i;
   unsigned char data;
   char datastr[3];
   FILE* pipefd = NULL;
 
-  memset(at_command,0,sizeof(at_command));
   memset(datastr,0,sizeof(datastr));
 
-  //creating at command
-  new_at_command(at_command,"^SBNR");
-  add_at_command(at_command,"=\"%s\",%d",ftype,slot);
-
-  //sending at command (parsing the answer)
-  tty_write_command(at_command);
+  at_sie_binary_read(ftype,slot);
   myprintf(0,"Slot %d...\n",slot);
   do{
-    ausgabe=tty_read(at_command);
+    ausgabe = at_read_line();
     //here we have to check not only for an output of OK, but because of
     //funny behaviour of the S55 and later, also for "+CME ERROR: unknown".
     //both stand for an empty entry
     if (!(strcmp(ausgabe,"OK")) ||
 	!(strcmp(ausgabe,"+CME ERROR: unknown"))) {
       mem_realloc(ausgabe,0);
-      myprintf(0,"Empty slot: nothing to get\n");
+      myprintf(0,"%s","Empty slot: nothing to get\n");
       close_myFile(myfd);
       return;
     } else if (strstr(ausgabe,"ERROR")!=NULL) {
@@ -438,7 +422,7 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
       }
       myprintf(0,"Receiving packet %d of %d...\n",packetnum,maxpackets);
       mem_realloc(ausgabe,0);
-      ausgabe=tty_read(at_command);
+      ausgabe = at_read_line();
       for (i=0;i<strlen(ausgabe);i+=2) {	
 	data=hexstr2int(ausgabe+i,2);
 	if (myfd!=-1) {
@@ -466,10 +450,10 @@ void file_get (char* file, char* ftype, int slot, char* pipe) {
     close(myfd);
     errexit("ERROR on closing pipe \"%s\".\n",pipe);			
   }
-  ausgabe=tty_read(at_command);
+  ausgabe = at_read_line();
   if (!(strcmp(ausgabe,"OK"))) {
     mem_realloc(ausgabe,0);
-    myprintf(0,"File transfer complete.\n");
+    myprintf(0,"%s","File transfer complete.\n");
   } else {
     errexit("%s, aborting\n",ausgabe);
   }
