@@ -1,3 +1,4 @@
+
 /***************************************************************************
  *   copyright           : (C) 2002 by Hendrik Sattler                     *
  *   mail                : post@hendrik-sattler.de                         *
@@ -10,8 +11,10 @@
  ***************************************************************************/
 
 #include "common.h"
-#include "helpers.h"
+#include "helper.h"
 #include "options.h"
+#include "atcommand.h"
+#include "ttyaccess.h"
 
 #include <locale.h>
 #include <string.h>
@@ -20,7 +23,7 @@
 int main (int argc, char** argv) {
   char* supported_phones[] = {
     "S25",
-    "S35i", "M35i", "C35i",
+    "S35i", "M35i", "MC35i", "C35i",
     "SL42", "SLIN", "SL45", "SLIK",
     "C45",  "ME45", "S45",  "S45i",
     "M50",  "MT50", "M50I",
@@ -38,9 +41,7 @@ int main (int argc, char** argv) {
 
   //start setup
   //init global vars	
-  PIN = NULL;
   VERBOSE_LEVEL = 0;
-  mytty = -1;
 
   //init local vars    
   vendor = NULL;
@@ -51,7 +52,7 @@ int main (int argc, char** argv) {
 
   if (argc==1) {
     help(argv[0]);
-    exit(0);
+    exit(EXIT_SUCCESS);
   }
 
   args_init(&args);
@@ -59,7 +60,6 @@ int main (int argc, char** argv) {
   use_envvars(&args);
 
   VERBOSE_LEVEL=args.verbose;
-  PIN = args.pin;
 
   if (args.help || args.version) {
     if (args.help) {
@@ -68,30 +68,32 @@ int main (int argc, char** argv) {
     if (args.version) {
       version();
     }
-    exit(0);
+    exit(EXIT_SUCCESS);
   }
 
   //now open the port
   tty_open(&args.port);
 
+  at_init(args.pin);
+
   if (args.reset) {
     /* sending a sync frame
      * the phone may not respond without it because it thinks that all
      * send commands are still data from a previous run
      */
-    myprintf(0,"Setting reset frame...");
-    tty_write("\x1a\r\n",3,"");
-    myprintf(0,"done\n");
-    exit(0);
+    myprintf(0,"%s","Setting reset frame...");
+    tty_write("\x1a\r\n",3);
+    myprintf(0,"%s","done\n");
+    exit(EXIT_SUCCESS);
   }
 
   /* Now perform initial phone setup
    */
   //testing the device
   if (phone_init()) {
-    myprintf(0,"OK, a modem device is present.\n");
+    myprintf(0,"%s","OK, a modem device is present.\n");
   } else {
-    errexit("ERROR, cannot communicate with device.\n");
+    errexit("%s","ERROR, cannot communicate with device.\n");
   }
   //disabling command echo
   command_echo(0);
@@ -118,10 +120,10 @@ int main (int argc, char** argv) {
 	  }
 	}
 	if (supported_phones[i] == NULL) {
-	  myprintf(0,"WARNING: this phone was not confirmed to be working with this software.\n");
+	  myprintf(1,"%s","WARNING: this phone was not confirmed to be working with this software.\n");
 	}
       } else {
-	myprintf(0,"WARNING: Phones from this vendor were not confirmed to be working with this software!\n");
+	myprintf(0,"%s","WARNING: Phones from this vendor were not confirmed to be working with this software!\n");
       }
     }
     mem_realloc(vendor,0);
@@ -137,18 +139,18 @@ int main (int argc, char** argv) {
       if (str_len(args.myparams.number)) {
 	set_smsc(args.myparams.number);
       } else {
-	errexit("You must define a number with the --number option.\n");
+	errexit("%s","You must define a number with the --number option.\n");
       }
     }
     if (args.info || args.meminfo) {
       info(args.myFILE,args.info,args.meminfo);
     }
-    exit(0);
+    exit(EXIT_SUCCESS);
   }
 
   switch(args.scmxx_ftype) {
   default:
-    errexit("You must specifiy a valid file type.\n");
+    errexit("%s","You must specifiy a valid memory type.\n");
     break;
   case SCMXX_FTYPE_BINARY:
     file_transfer_manage(args.scmxx_action,files,
@@ -158,7 +160,7 @@ int main (int argc, char** argv) {
     break;
   case SCMXX_FTYPE_PBOOK:
     pbook_manage(args.scmxx_action,args.myFILE,
-		 files,args.myparams.mem);
+		 files,&args.myparams);
     break;
   case SCMXX_FTYPE_SMS:
     sms_manage(args.scmxx_action,files,
@@ -167,23 +169,5 @@ int main (int argc, char** argv) {
     break;
   }
   tty_close();
-  exit(0);
-}
-
-void errexit(char *errmessage, ...){
-  va_list arg_list;
-  va_start(arg_list,errmessage);
-  vfprintf(stderr,errmessage,arg_list);
-  va_end(arg_list);
-  tty_close();
-  exit(1);
-}
-
-void myprintf(int verbose_level, char *output, ...){
-  va_list arg_list;
-  va_start(arg_list,output);
-  if (verbose_level<=VERBOSE_LEVEL) {
-    vfprintf(stderr,output,arg_list);
-  }
-  va_end(arg_list);
+  exit(EXIT_SUCCESS);
 }
